# Generated by Django 5.2.7 on 2025-11-19 18:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("order", "0015_alter_line_num_allocated"),
    ]

    operations = [
        migrations.AddField(
            model_name="order",
            name="analytics_tracked",
            field=models.BooleanField(default=False),
        ),
    ]
