# Generated by Django 3.1.6 on 2021-02-01 17:35

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("order", "0009_surcharge"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="orderdiscount",
            options={
                "ordering": ["pk"],
                "verbose_name": "Order Discount",
                "verbose_name_plural": "Order Discounts",
            },
        ),
        migrations.AlterModelOptions(
            name="ordernote",
            options={
                "ordering": ["-date_updated"],
                "verbose_name": "Order Note",
                "verbose_name_plural": "Order Notes",
            },
        ),
        migrations.RemoveField(
            model_name="line",
            name="est_dispatch_date",
        ),
        migrations.RemoveField(
            model_name="line",
            name="unit_cost_price",
        ),
        migrations.RemoveField(
            model_name="line",
            name="unit_retail_price",
        ),
    ]
