# Generated by Django 2.2.6 on 2020-02-19 09:16

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("order", "0008_auto_20190301_1035"),
    ]

    replaces = [
        ("order", "0008_surcharge"),
    ]

    operations = [
        migrations.CreateModel(
            name="Surcharge",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=128, verbose_name="Name")),
                (
                    "code",
                    models.CharField(max_length=128, verbose_name="Surcharge code"),
                ),
                (
                    "incl_tax",
                    models.DecimalField(
                        decimal_places=2,
                        default=0,
                        max_digits=12,
                        verbose_name="Surcharge (inc. tax)",
                    ),
                ),
                (
                    "excl_tax",
                    models.DecimalField(
                        decimal_places=2,
                        default=0,
                        max_digits=12,
                        verbose_name="Surcharge (excl. tax)",
                    ),
                ),
                (
                    "order",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="surcharges",
                        to="order.Order",
                        verbose_name="Order",
                    ),
                ),
            ],
            options={"abstract": False, "ordering": ["pk"]},
        ),
    ]
