# Generated by Django 4.2.6 on 2023-10-26 16:21

from django.db import migrations, models
import django.core.serializers.json


class Migration(migrations.Migration):
    dependencies = [
        ("order", "0010_auto_20210201_1735"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="surcharge",
            options={
                "ordering": ["pk"],
                "verbose_name": "Surcharge",
                "verbose_name_plural": "Surcharges",
            },
        ),
        migrations.AddField(
            model_name="line",
            name="tax_code",
            field=models.CharField(
                blank=True, max_length=64, null=True, verbose_name="VAT rate code"
            ),
        ),
        migrations.AddField(
            model_name="lineprice",
            name="tax_code",
            field=models.CharField(
                blank=True, max_length=64, null=True, verbose_name="VAT rate code"
            ),
        ),
        migrations.AddField(
            model_name="order",
            name="shipping_tax_code",
            field=models.CharField(
                blank=True,
                max_length=64,
                null=True,
                verbose_name="Shipping VAT rate code",
            ),
        ),
        migrations.AddField(
            model_name="surcharge",
            name="tax_code",
            field=models.CharField(
                blank=True, max_length=64, null=True, verbose_name="VAT rate code"
            ),
        ),
        migrations.AlterField(
            model_name="lineattribute",
            name="value",
            field=models.JSONField(
                encoder=django.core.serializers.json.DjangoJSONEncoder,
                verbose_name="Value",
            ),
        ),
    ]
